@@ -4,9 +4,15 @@ libEnsemble.
 Library to coordinate the concurrent evaluation of dynamic ensembles of calculations.
 """
 
-__version__ = "0.7.0"
+from .version import __version__
 __author__ = 'Jeffrey Larson, Stephen Hudson, Stefan M. Wild, David Bindel and John-Luke Navarro'
 __credits__ = 'Argonne National Laboratory'
 
-from libensemble import libE_logger
+from libensemble import logger
+
 from .libE import libE, comms_abort
+
+try:
+    from .api import Ensemble
+except ModuleNotFoundError:
+    pass
