@@ -32,7 +32,7 @@ int main(int argc, char **argv)
 
     if (rank==0) {
         if (error==1) {
-            printf("Oh Dear! An non-fatal Error seems to have occured on rank %d\n",rank);
+            printf("Oh Dear! An non-fatal Error seems to have occurred on rank %d\n",rank);
             sleep(delay);
         }
     }
