@@ -13,69 +13,73 @@ if tuple(np.__version__.split('.')) >= ('1', '15'):
 # - compare selected values
 # - compare from npy file - stored
 
-wrs_H0 = np.array([(False, 0., 0, 0., 1, True, 1, True, [0., 0., 0.], True, 0.1, 1.1, False, False),
-                   (False, 0., 0, 0., 1, True, 2, True, [0., 0., 0.], True, 0.2, 1.2, False, False),
-                   (False, 0., 0, 0., 1, True, 3, True, [0., 0., 0.], True, 0.3, 1.3, False, False)],
+wrs_H0 = np.array([(False, 0., 0, 0., 1, True, 1, True, [0., 0., 0.], True, 0.1, 1.1, False, False, False, inf),
+                   (False, 0., 0, 0., 1, True, 2, True, [0., 0., 0.], True, 0.2, 1.2, False, False, False, inf),
+                   (False, 0., 0, 0., 1, True, 3, True, [0., 0., 0.], True, 0.3, 1.3, False, False, False, inf)],
                   dtype=[('local_pt', '?'), ('priority', '<f8'), ('gen_worker', '<i8'), ('x_on_cube', '<f8'),
                          ('sim_id', '<i8'), ('given', '?'), ('sim_worker', '<i8'), ('returned', '?'),
                          ('fvec', '<f8', (3,)), ('allocated', '?'), ('f', '<f8'), ('given_time', '<f8'),
-                         ('cancel_requested', '?'), ('kill_sent', '?')])
+                         ('cancel_requested', '?'), ('kill_sent', '?'), ('given_back', '?'), ('last_given_back_time', '<f8')])
 
-exp_H0_H = np.array([(False, 0., 0, 0., 1, True, 1, True, [0., 0., 0.], True, 0.1, 1.1, 2.1, False, False),
-                     (False, 0., 0, 0., 1, True, 2, True, [0., 0., 0.], True, 0.2, 1.2, 2.3, False, False),
-                     (False, 0., 0, 0., 1, True, 3, True, [0., 0., 0.], True, 0.3, 1.3, 2.3, False, False),
-                     (False, 0., 0, 0., -1, False, 0, False, [0., 0., 0.], False, 0., inf, 0., False, False),
-                     (False, 0., 0, 0., -1, False, 0, False, [0., 0., 0.], False, 0., inf, 0., False, False)],
+exp_H0_H = np.array([(False, 0., 0, 0., 1, True, 1, True, [0., 0., 0.], True, 0.1, 1.1, 2.1, False, False, False, inf),
+                     (False, 0., 0, 0., 1, True, 2, True, [0., 0., 0.], True, 0.2, 1.2, 2.3, False, False, False, inf),
+                     (False, 0., 0, 0., 1, True, 3, True, [0., 0., 0.], True, 0.3, 1.3, 2.3, False, False, False, inf),
+                     (False, 0., 0, 0., -1, False, 0, False, [0., 0., 0.], False, 0., inf, 0., False, False, False, inf),
+                     (False, 0., 0, 0., -1, False, 0, False, [0., 0., 0.], False, 0., inf, 0., False, False, False, inf)],
                     dtype=[('local_pt', '?'), ('priority', '<f8'), ('gen_worker', '<i8'), ('x_on_cube', '<f8'),
                            ('sim_id', '<i8'), ('given', '?'), ('sim_worker', '<i8'), ('returned', '?'),
                            ('fvec', '<f8', (3,)), ('allocated', '?'), ('f', '<f8'), ('given_time', '<f8'),
-                           ('returned_time', '<f8'), ('cancel_requested', '?'), ('kill_sent', '?')])
+                           ('returned_time', '<f8'), ('cancel_requested', '?'), ('kill_sent', '?'),
+                           ('given_back', '?'), ('last_given_back_time', '<f8')])
 
-wrs = np.array([(False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False),
-                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False),
-                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False),
-                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False),
-                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False),
-                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False),
-                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False),
-                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False),
-                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False),
-                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False)],
+wrs = np.array([(False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False, False, inf),
+                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False, False, inf),
+                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False, False, inf),
+                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False, False, inf),
+                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False, False, inf),
+                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False, False, inf),
+                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False, False, inf),
+                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False, False, inf),
+                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False, False, inf),
+                (False, 0., 0, 0., 0., 0., -1, False, 0, False, 0, [0., 0., 0.], False, 0., inf, inf, False, 0, False, False, False, inf)],
                dtype=[('local_pt', '?'), ('priority', '<f8'), ('gen_worker', '<i8'), ('gen_time', '<f8'),
                       ('last_gen_time', '<f8'), ('x_on_cube', '<f8'), ('sim_id', '<i8'), ('given', '?'),
                       ('sim_worker', '<i8'), ('returned', '?'), ('returned_time', '<f8'), ('fvec', '<f8', (3,)),
                       ('allocated', '?'), ('f', '<f8'), ('given_time', '<f8'), ('last_given_time', '<f8'),
-                      ('local_min', '?'), ('num_active_runs', '<i8'), ('cancel_requested', '?'), ('kill_sent', '?')])
+                      ('local_min', '?'), ('num_active_runs', '<i8'), ('cancel_requested', '?'), ('kill_sent', '?'),
+                      ('given_back', '?'), ('last_given_back_time', '<f8')])
 
-wrs2 = np.array([(0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False),
-                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False),
-                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False),
-                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False),
-                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False),
-                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False),
-                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False),
-                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False),
-                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False),
-                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False)],
+wrs2 = np.array([(0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False, False, inf),
+                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False, False, inf),
+                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False, False, inf),
+                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False, False, inf),
+                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False, False, inf),
+                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False, False, inf),
+                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False, False, inf),
+                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False, False, inf),
+                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False, False, inf),
+                 (0, False, 0., 0., 0., 0., 0, False, 0., False, -1, inf, inf, 0., False, False, False, inf)],
                 dtype=[('gen_worker', '<i8'), ('returned', '?'), ('returned_time', '<f8'),
                        ('gen_time', '<f8'), ('last_gen_time', '<f8'), ('x', '<f8'),
                        ('sim_worker', '<i8'), ('allocated', '?'), ('g', '<f8'), ('given', '?'),
                        ('sim_id', '<i8'), ('given_time', '<f8'), ('last_given_time', '<f8'),
-                       ('priority', '<f8'), ('cancel_requested', '?'), ('kill_sent', '?')])
+                       ('priority', '<f8'), ('cancel_requested', '?'), ('kill_sent', '?'),
+                       ('given_back', '?'), ('last_given_back_time', '<f8')])
 
-exp_x_in_setup2 = np.array([(0, 0, 2, 0., 4.17022005e-01, False, False, False, inf, 0., False, False),
-                            (0, 1, 3, 0., 7.20324493e-01, False, False, False, inf, 0., False, False),
-                            (0, 2, 3, 0., 1.14374817e-04, False, False, False, inf, 0., False, False),
-                            (0, 3, 3, 0., 3.02332573e-01, False, False, False, inf, 0., False, False),
-                            (0, 4, 3, 0., 1.46755891e-01, False, False, False, inf, 0., False, False),
-                            (0, 5, 3, 0., 9.23385948e-02, False, False, False, inf, 0., False, False),
-                            (0, 6, 3, 0., 1.86260211e-01, False, False, False, inf, 0., False, False),
-                            (0, 7, 3, 0., 3.45560727e-01, False, False, False, inf, 0., False, False),
-                            (0, 8, 3, 0., 3.96767474e-01, False, False, False, inf, 0., False, False),
-                            (0, 9, 3, 0., 5.38816734e-01, False, False, False, inf, 0., False, False)],
+exp_x_in_setup2 = np.array([(0, 0, 2, 0., 4.17022005e-01, False, False, False, inf, 0., False, False, False, inf),
+                            (0, 1, 3, 0., 7.20324493e-01, False, False, False, inf, 0., False, False, False, inf),
+                            (0, 2, 3, 0., 1.14374817e-04, False, False, False, inf, 0., False, False, False, inf),
+                            (0, 3, 3, 0., 3.02332573e-01, False, False, False, inf, 0., False, False, False, inf),
+                            (0, 4, 3, 0., 1.46755891e-01, False, False, False, inf, 0., False, False, False, inf),
+                            (0, 5, 3, 0., 9.23385948e-02, False, False, False, inf, 0., False, False, False, inf),
+                            (0, 6, 3, 0., 1.86260211e-01, False, False, False, inf, 0., False, False, False, inf),
+                            (0, 7, 3, 0., 3.45560727e-01, False, False, False, inf, 0., False, False, False, inf),
+                            (0, 8, 3, 0., 3.96767474e-01, False, False, False, inf, 0., False, False, False, inf),
+                            (0, 9, 3, 0., 5.38816734e-01, False, False, False, inf, 0., False, False, False, inf)],
                            dtype=[('sim_worker', '<i8'), ('sim_id', '<i8'), ('gen_worker', '<i8'), ('priority', '<f8'),
                                   ('x', '<f8'), ('allocated', '?'), ('returned', '?'), ('given', '?'),
-                                  ('given_time', '<f8'), ('g', '<f8'), ('cancel_requested', '?'), ('kill_sent', '?')])
+                                  ('given_time', '<f8'), ('g', '<f8'), ('cancel_requested', '?'), ('kill_sent', '?'),
+                                  ('given_back', '?'), ('last_given_back_time', '<f8')])
 
 safe_mode = True
 
@@ -90,7 +94,8 @@ def test_hist_init_1():
     assert np.array_equal(hist.H, wrs), "Array does not match expected"
     assert hist.given_count == 0
     assert hist.index == 0
-    assert hist.sim_count == 0
+    assert hist.returned_count == 0
+    assert hist.given_back_count == 0
 
 
 def test_hist_init_1A_H0():
@@ -104,7 +109,8 @@ def test_hist_init_1A_H0():
     # assert np.array_equal(hist.H, exp_H0_H), "Array does not match expected"
     assert hist.given_count == 3
     assert hist.index == 3
-    assert hist.sim_count == 3
+    assert hist.returned_count == 3
+    assert hist.given_back_count == 0
     assert len(hist.H) == 5
 
 
@@ -113,7 +119,8 @@ def test_hist_init_2():
     assert np.array_equal(hist.H, wrs2), "Array does not match expected"
     assert hist.given_count == 0
     assert hist.index == 0
-    assert hist.sim_count == 0
+    assert hist.returned_count == 0
+    assert hist.given_back_count == 0
 
 
 def test_grow_H():
@@ -123,7 +130,8 @@ def test_grow_H():
     assert np.array_equal(hist.H, wrs), "Array does not match expected"
     assert hist.given_count == 0
     assert hist.index == 0
-    assert hist.sim_count == 0
+    assert hist.returned_count == 0
+    assert hist.given_back_count == 0
 
 
 def test_trim_H():
@@ -133,7 +141,8 @@ def test_trim_H():
     assert np.array_equal(H, wrs), "Array does not match expected"
     assert hist.given_count == 0
     assert hist.index == 10
-    assert hist.sim_count == 0
+    assert hist.returned_count == 0
+    assert hist.given_back_count == 0
 
 
 def test_update_history_x_in_Oempty():
@@ -144,7 +153,8 @@ def test_update_history_x_in_Oempty():
     assert np.array_equal(hist.H, wrs2), "H Array does not match expected"
     assert hist.given_count == 0
     assert hist.index == 0
-    assert hist.sim_count == 0
+    assert hist.returned_count == 0
+    assert hist.given_back_count == 0
 
 
 def test_update_history_x_in():
@@ -165,7 +175,8 @@ def test_update_history_x_in():
     assert isclose(single_rand, hist.H['x'][0])
     assert hist.given_count == 0
     assert hist.index == 1
-    assert hist.sim_count == 0
+    assert hist.returned_count == 0
+    assert hist.given_back_count == 0
 
     size = 6
     gen_worker = 3
@@ -180,7 +191,8 @@ def test_update_history_x_in():
 
     assert hist.given_count == 0
     assert hist.index == 7
-    assert hist.sim_count == 0
+    assert hist.returned_count == 0
+    assert hist.given_back_count == 0
 
     # Force H to grow when add points
     size = 3
@@ -196,7 +208,16 @@ def test_update_history_x_in():
 
     assert hist.given_count == 0
     assert hist.index == 10
-    assert hist.sim_count == 0
+    assert hist.returned_count == 0
+
+    # Test libE errors when a protected field appears in output from a gen_worker
+    H_o = np.zeros(size, dtype=gen_specs['out'] + [('given', bool)])
+    try:
+        hist.update_history_x_in(gen_worker, H_o, safe_mode)
+    except AssertionError:
+        assert 1, "Failed like it should have"
+    else:
+        assert 0, "Didn't fail like it should have"
 
     # Test libE errors when a protected field appears in output from a gen_worker
     H_o = np.zeros(size, dtype=gen_specs['out'] + [('given', bool)])
@@ -227,7 +248,8 @@ def test_update_history_x_in_sim_ids():
     assert isclose(single_rand, hist.H['x'][0])
     assert hist.given_count == 0
     assert hist.index == 1
-    assert hist.sim_count == 0
+    assert hist.returned_count == 0
+    assert hist.given_back_count == 0
 
     size = 6
     gen_worker = 3
@@ -243,7 +265,8 @@ def test_update_history_x_in_sim_ids():
 
     assert hist.given_count == 0
     assert hist.index == 7
-    assert hist.sim_count == 0
+    assert hist.returned_count == 0
+    assert hist.given_back_count == 0
 
     # Force H to grow when add points
     size = 3
@@ -260,7 +283,8 @@ def test_update_history_x_in_sim_ids():
 
     assert hist.given_count == 0
     assert hist.index == 10
-    assert hist.sim_count == 0
+    assert hist.returned_count == 0
+    assert hist.given_back_count == 0
 
 
 # Note - Ideally have more setup here (so hist.index reflects generated points)
@@ -277,7 +301,7 @@ def test_update_history_x_out():
 
     # Check some unchanged values for point and counts
     assert hist.index == 0
-    assert hist.sim_count == 0
+    assert hist.returned_count == 0
     hist.H['returned'][0] = False
     hist.H['allocated'][0] = False
     hist.H['f'][0] == 0.0
@@ -307,7 +331,8 @@ def test_update_history_x_out():
     # Check counts
     assert hist.given_count == 6
     assert hist.index == 0  # In real case this would be ahead.....
-    assert hist.sim_count == 0
+    assert hist.returned_count == 0
+    assert hist.given_back_count == 0
 
 
 def test_update_history_f():
@@ -316,8 +341,7 @@ def test_update_history_f():
 
     # First update a single point
     size = 1
-    sim_ids = 0  # First row to be filled
-    sim_ids = np.atleast_1d(sim_ids)
+    sim_ids = [0]  # First row to be filled
     calc_out = np.zeros(size, dtype=sim_specs['out'])
     a = np.arange(9) - 4
     calc_out['g'] = sim_specs['sim_f'](a)  # np.linalg.norm
@@ -332,14 +356,13 @@ def test_update_history_f():
     assert isclose(exp_vals[0], hist.H['g'][0])
     assert np.all(hist.H['returned'][0:1])
     assert np.all(~hist.H['returned'][1:10])  # Check the rest
-    assert hist.sim_count == 1
+    assert hist.returned_count == 1
     assert hist.given_count == 0  # In real case this would be ahead.....
     assert hist.index == 0  # In real case this would be ahead....
 
     # Update two further consecutive points
     size = 2
     sim_ids = [1, 2]  # First row to be filled
-    sim_ids = np.atleast_1d(sim_ids)
     calc_out = np.zeros(size, dtype=sim_specs['out'])
     a = np.arange(9) - 3
     calc_out['g'][0] = sim_specs['sim_f'](a)  # np.linalg.norm
@@ -357,7 +380,7 @@ def test_update_history_f():
     assert np.allclose(exp_vals, hist.H['g'])
     assert np.all(hist.H['returned'][0:3])
     assert np.all(~hist.H['returned'][3:10])  # Check the rest
-    assert hist.sim_count == 3
+    assert hist.returned_count == 3
     assert hist.given_count == 0  # In real case this would be ahead.....
     assert hist.index == 0  # In real case this would be ahead....
 
@@ -369,8 +392,7 @@ def test_update_history_f_vec():
 
     # First update a single point
     size = 1
-    sim_ids = 0  # First row to be filled
-    sim_ids = np.atleast_1d(sim_ids)
+    sim_ids = [0]  # First row to be filled
     calc_out = np.zeros(size, dtype=sim_specs['out'])
     a = np.array([[1, 2, 3], [-1, 1, 4]])
     calc_out['f'] = sim_specs['sim_f'](a)  # np.linalg.norm
@@ -389,14 +411,13 @@ def test_update_history_f_vec():
     assert np.allclose(exp_fvecs[0], hist.H['fvec'][0])
     assert np.all(hist.H['returned'][0:1])
     assert np.all(~hist.H['returned'][1:10])  # Check the rest
-    assert hist.sim_count == 1
+    assert hist.returned_count == 1
     assert hist.given_count == 0  # In real case this would be ahead.....
     assert hist.index == 0  # In real case this would be ahead....
 
     # Update two further consecutive points
     size = 2
     sim_ids = [1, 2]  # First row to be filled
-    sim_ids = np.atleast_1d(sim_ids)
     calc_out = np.zeros(size, dtype=sim_specs['out'])
 
     a = np.array([[1, 3, 4], [-1, 2, 4]])
@@ -423,14 +444,13 @@ def test_update_history_f_vec():
     assert np.allclose(exp_fvecs, hist.H['fvec'])
     assert np.all(hist.H['returned'][0:3])
     assert np.all(~hist.H['returned'][3:10])  # Check the rest
-    assert hist.sim_count == 3
+    assert hist.returned_count == 3
     assert hist.given_count == 0  # In real case this would be ahead.....
     assert hist.index == 0  # In real case this would be ahead....
 
-    # Update two further consecutive points but with sub_array fvec componenets
+    # Update two further consecutive points but with sub_array fvec components
     size = 2
     sim_ids = [3, 4]  # First row to be filled
-    sim_ids = np.atleast_1d(sim_ids)
     calc_out = np.zeros(size, dtype=[('f', float), ('fvec', float, 2)])  # Only two values
 
     a = np.array([[1, 3, 4], [-1, 2, 4]])
@@ -459,7 +479,7 @@ def test_update_history_f_vec():
     assert np.allclose(exp_fvecs, hist.H['fvec'])
     assert np.all(hist.H['returned'][0:5])
     assert np.all(~hist.H['returned'][5:10])  # Check the rest
-    assert hist.sim_count == 5
+    assert hist.returned_count == 5
     assert hist.given_count == 0  # In real case this would be ahead.....
     assert hist.index == 0  # In real case this would be ahead....
 
